@@ -1,7 +1,6 @@
 #-*- coding:utf-8 -*-
 
-#  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009-2013  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2015  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -24,10 +23,12 @@ import sys
 
 DEBUG = False
 DEBUG_FUNC = DEBUG_MSG = DEBUG_RAND = DEBUG_ROTATE = False
-DEBUG_DRAW = DEBUG_KEYS = DEBUG_PICK = False
-DEBUG_FPS = DEBUG_VFPS = DEBUG_NOLABEL = DEBUG_NOBEVEL = DEBUG_INNER = False
-DEBUG_ALG = DEBUG_NOVSHADER = DEBUG_NOFSHADER = DEBUG_NOSHADER = False
-DEBUG_THREAD = DEBUG_PUREPYTHON = False
+DEBUG_DRAW = DEBUG_ALG = DEBUG_KEYS = DEBUG_PICK = DEBUG_MOUSEPOS = False
+DEBUG_FPS = DEBUG_VFPS = DEBUG_NOLABEL = DEBUG_NOBEVEL = DEBUG_INVISIBLE = False
+DEBUG_SHADER = DEBUG_NOVSHADER = DEBUG_NOFSHADER = DEBUG_NOSHADER = False
+DEBUG_PUREPYTHON = DEBUG_INDEXONLY = DEBUG_LIVESHADER = False
+DEBUG_LOG = DEBUG_LOGDEBUG = DEBUG_LOGGL = False
+DEBUG_LIVEPLUGINS = False
 
 __all__ = ['error', 'debug', 'debug_func', 'DEBUG',
           ] + [__n for __n in dir(sys.modules[__name__]) if __n.startswith('DEBUG_')]
@@ -45,6 +46,12 @@ def set_flags(debug_flags):
     for flag in debug_flags:
         setattr(module, 'DEBUG_' + flag.upper(), True)
         
+    if module.DEBUG_LOGDEBUG or module.DEBUG_LOGGL:
+        module.DEBUG_LOG = True
+    if module.DEBUG_LOG:
+        import logging
+        logging.basicConfig(filename='pybik.log',
+                            level=logging.DEBUG if module.DEBUG_LOGDEBUG else logging.INFO)
     if module.DEBUG_VFPS:
         module.DEBUG_FPS = True
     
