@@ -1,7 +1,6 @@
 #-*- coding:utf-8 -*-
 
-#  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009, 2011-2013  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009, 2011-2015  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,27 +15,25 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Ported from GNUbik
-# Original filename: menus.c
-# Original copyright and license: 2003, 2004  John Darrington,  GPL3+
 
 # pylint: disable=C0321
 
-
 import os
 import re
+import html
+from itertools import zip_longest
 
-# pylint: disable=W0614,W0401
-from PyQt4.QtCore import *
-from PyQt4.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
-from PyQt4.QtGui import *
+from PyQt5.QtCore import QAbstractAnimation, QEvent, QObject, QPropertyAnimation, QSize, QStandardPaths, Qt, QTimer
+from PyQt5.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
+from PyQt5.QtGui import (QColor, QDesktopServices, QFontMetrics, QIcon, QIconEngine, QKeySequence, QPixmap,
+                         QStandardItem, QStandardItemModel, QTextDocumentFragment)
+from PyQt5.QtWidgets import (QAbstractItemDelegate, QColorDialog, QDialog, QDialogButtonBox, QFileDialog, QLabel,
+                         QProgressDialog, QPushButton, QStyledItemDelegate, QTextEdit)
 
-from .debug import *
-# pylint: enable=W0614,W0401
 from . import config
 from .settings import settings
-from .textures import textures
-from .model import models
+from .theme import Theme
+from .model import Model
 
 try:
     _
@@ -48,17 +45,18 @@ class Dialog (QDialog):
     _instance = None
     _classes = []
     
-    def __init__(self, parent):
-        QDialog.__init__(self, parent)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        
         self.accepted.connect(self.on_response)
         self._ignore_changed = False
         self.bound = {}
         settings.keystore.changed.connect(self.on_settings_changed, Qt.QueuedConnection)
         
     @classmethod
-    def run(cls, parent_window=None):
+    def run(cls, **kwargs):
         if cls._instance is None:
-            cls._instance = cls(parent_window)
+            cls._instance = cls(**kwargs)
             ret = cls._instance
             cls._classes.append(cls)
         else:
@@ -116,9 +114,10 @@ class Dialog (QDialog):
         self._ignore_changed = False
         
         
-class ColorIconEngine (QIconEngineV2):
-    def __init__(self):
-        QIconEngineV2.__init__(self)
+class ColorIconEngine (QIconEngine):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        
         self.color = 'black'
         
     def paint(self, painter, rect, unused_mode, unused_state):
@@ -128,10 +127,11 @@ class ColorIconEngine (QIconEngineV2):
 class ColorButton (QPushButton):
     color_changed = Signal(str)
     
-    def __init__(self, replace_button):
+    def __init__(self, replace_button, **kwargs):
         self._icon = ColorIconEngine()
         parent = replace_button.parentWidget()
-        QPushButton.__init__(self, QIcon(self._icon), '', parent)
+        super().__init__(QIcon(self._icon), '', parent, **kwargs)
+        
         height = self.iconSize().height()
         self.setIconSize(QSize(height * self.width() // self.height(), height))
         self.setSizePolicy(replace_button.sizePolicy())
@@ -164,8 +164,9 @@ class ColorButton (QPushButton):
 class ShortcutEditor (QLabel):
     key_pressed = Signal()
     
-    def __init__(self, parent):
-        QLabel.__init__(self, _('Press a key …'), parent)
+    def __init__(self, **kwargs):
+        super().__init__(text=_('Press a key …'), **kwargs)
+        
         self.setFocusPolicy(Qt.StrongFocus)
         self.setAutoFillBackground(True)
         self.key = None
@@ -178,7 +179,7 @@ class ShortcutEditor (QLabel):
         if event.key() in self.IGNORE_KEYS or event.count() != 1:
             return QLabel.keyPressEvent(self, event)
         mod = event.modifiers() & self.SAFE_MODIFIER_MASK
-        key = QKeySequence(event.key() | mod).toString().split('+')
+        key = QKeySequence(event.key() | int(mod)).toString().split('+')
         if  event.modifiers() & Qt.KeypadModifier:
             key.insert(-1, 'KP')
         self.key = '+'.join(key)
@@ -186,11 +187,11 @@ class ShortcutEditor (QLabel):
         
         
 class ShortcutDelegate (QStyledItemDelegate):
-    def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
         
     def createEditor(self, parent, unused_option, unused_index):
-        editor = ShortcutEditor(parent)
+        editor = ShortcutEditor(parent=parent)
         editor.key_pressed.connect(self.on_editor_key_pressed)
         return editor
         
@@ -210,14 +211,15 @@ class ShortcutDelegate (QStyledItemDelegate):
 class PreferencesDialog (Dialog):
     sample_buffers = 0
         
-    def __init__(self, parent):
-        Dialog.__init__(self, parent)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        
         self.ui = UI(self, 'preferences.ui')
         from .ui.preferences import retranslate
-        retranslate(self)
+        self.ui = retranslate(self)
         
         self.ui.button_animspeed_reset.setIcon(QIcon.fromTheme('edit-clear'))
-        self.ui.button_lighting_reset.setIcon(QIcon.fromTheme('edit-clear'))
+        self.ui.button_shader_reset.setIcon(QIcon.fromTheme('edit-clear'))
         self.ui.button_antialiasing_reset.setIcon(QIcon.fromTheme('edit-clear'))
         self.ui.button_mirror_faces_reset.setIcon(QIcon.fromTheme('edit-clear'))
         self.ui.button_movekey_add.setIcon(QIcon.fromTheme('list-add'))
@@ -226,8 +228,8 @@ class PreferencesDialog (Dialog):
         self.ui.button_color_reset.setIcon(QIcon.fromTheme('edit-clear'))
         self.ui.button_image_reset.setIcon(QIcon.fromTheme('edit-clear'))
         self.ui.button_background_color_reset.setIcon(QIcon.fromTheme('edit-clear'))
-        self.ui.button_mousemode_quad.setIcon(QIcon(os.path.join(config.UI_DIR, "mousemode-quad.png")))
-        self.ui.button_mousemode_ext.setIcon(QIcon(os.path.join(config.UI_DIR, "mousemode-ext.png")))
+        self.ui.button_mousemode_quad.setIcon(QIcon(os.path.join(config.UI_DIR, 'qt', 'mousemode-quad.png')))
+        self.ui.button_mousemode_ext.setIcon(QIcon(os.path.join(config.UI_DIR, 'qt', 'mousemode-ext.png')))
         
         self.ui.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
         self.ui.label_needs_restarted.setVisible(False)
@@ -238,17 +240,20 @@ class PreferencesDialog (Dialog):
         self.bind('draw.speed', self.ui.slider_animspeed, 'value', 'setValue', 'valueChanged')
         self.bind_reset('draw.speed', self.ui.button_animspeed_reset)
         
-        self.bind('draw.lighting', self.ui.checkbox_lighting, 'isChecked', 'setChecked', 'toggled')
-        self.bind_reset('draw.lighting', self.ui.button_lighting_reset)
+        shader_names = {'lighting': _('Lighting'), 'simple': _('Simple')}
+        for nick in settings.draw.shader_range:
+            self.ui.combobox_shader.addItem(shader_names.get(nick, nick), nick)
+        self.ui.combobox_shader.setCurrentIndex(settings.draw.shader)
+        self.bind('draw.shader', self.ui.combobox_shader,
+                                  'currentIndex', 'setCurrentIndex', 'currentIndexChanged')
+        self.bind_reset('draw.shader', self.ui.button_shader_reset)
         
-        def reset_antialiasing():
-            del settings.draw.samples
-        self.ui.button_antialiasing_reset.clicked.connect(reset_antialiasing)
         for text in settings.draw.samples_range:
             self.ui.combobox_samples.addItem(_(text), text)
         self.ui.combobox_samples.setCurrentIndex(settings.draw.samples)
         self.bind('draw.samples', self.ui.combobox_samples,
                                   'currentIndex', 'setCurrentIndex', 'currentIndexChanged')
+        self.bind_reset('draw.samples', self.ui.button_antialiasing_reset)
         
         self.ui.spinbox_mirror_faces.setRange(*settings.draw.mirror_distance_range)
         def set_mirror_faces(checked):
@@ -259,9 +264,9 @@ class PreferencesDialog (Dialog):
         self.bind('draw.mirror_distance', self.ui.spinbox_mirror_faces, 'value', 'setValue', 'valueChanged')
         self.bind_reset('draw.mirror_distance', self.ui.button_mirror_faces_reset)
         
-        html = '<html><head/><body><p><span style=" color:#aa0000;">{}</span></p></body></html>'
-        html = html.format(Qt.escape(_('The program needs to be restarted for the changes to take effect.')))
-        self.ui.label_needs_restarted.setText(html)
+        html_text = '<html><head/><body><p><span style=" color:#aa0000;">{}</span></p></body></html>'
+        html_text = html_text.format(html.escape(_('The program needs to be restarted for the changes to take effect.')))
+        self.ui.label_needs_restarted.setText(html_text)
         
         # mouse tab
         def set_selection_mode(unused_mode):
@@ -276,54 +281,58 @@ class PreferencesDialog (Dialog):
         self.fill_liststore_movekeys()
         self.ui.listview_movekeys.setModel(self.liststore_movekeys)
         self.liststore_movekeys.itemChanged.connect(self.on_liststore_movekeys_itemChanged)
-        self.shortcut_delegate = ShortcutDelegate(self.ui.listview_movekeys)
+        self.shortcut_delegate = ShortcutDelegate(parent=self.ui.listview_movekeys)
         self.ui.listview_movekeys.setItemDelegateForColumn(1, self.shortcut_delegate)
         
-        # design tab
-        self.image_dirname = os.environ.get('HOME', '') #TODO: Use Picture folder, Qt5 has a class for that
-        self.active_face = 0
-        self.liststore_faces = self.create_face_selector()
+        # theme tab
+        location = QStandardPaths.standardLocations(QStandardPaths.PicturesLocation)
+        if not location:
+            location = QStandardPaths.standardLocations(QStandardPaths.HomeLocation)
+            if not location:
+                location = ['']
+        self.image_dirname = location[0]
         
         self.button_color = ColorButton(self.ui.button_color)
-        self.ui.button_color = None
         self.setTabOrder(self.ui.listview_faces, self.button_color)
         self.button_color.color_changed.connect(self.on_button_color_color_changed)
         self.button_background_color = ColorButton(self.ui.button_background_color)
-        self.ui.button_background_color = None
         self.setTabOrder(self.ui.radiobutton_mosaic, self.button_background_color)
         
         # Only a single color, no picture or pattern
+        self.stockfiles = ['']
         self.ui.combobox_image.addItem(_('plain'), '')
-        for filename in textures.stock_files:
-            pixmap = QPixmap(textures.get_stock_pixbuf(filename))
-            icon = QIcon(pixmap)
+        for filename, icon in Theme.textures.get_icons():
+            self.stockfiles.append(filename)
             self.ui.combobox_image.addItem(icon, '', filename)
         self.ui.combobox_image.addItem(_('select …'), '/')
         
-        self.bind_reset_item('theme.face.{}.color', self.ui.button_color_reset)
-        self.bind_reset_item('theme.face.{}.image', self.ui.button_image_reset)
+        self.bind_reset_item('theme.faces.{}.color', self.ui.button_color_reset)
+        self.bind_reset_item('theme.faces.{}.image', self.ui.button_image_reset)
         self.bind('theme.bgcolor', self.button_background_color, 'get_color', 'set_color', 'color_changed')
         self.bind_reset('theme.bgcolor', self.ui.button_background_color_reset)
         
-        index = self.liststore_faces.index(self.active_face, 1)
-        self.ui.listview_faces.setCurrentIndex(index)
+        self.facenames = Model.cache_index['facenames']
+        self.liststore_faces = QStandardItemModel()
+        self.ui.listview_faces.setModel(self.liststore_faces)
+        self.ui.listview_faces.selectionModel().currentChanged.connect(self.on_listview_faces_currentChanged)
+        self.fill_face_selector()
         
-    def create_face_selector(self):
-        liststore_faces = QStandardItemModel()
-        face_names = [_('Up'), _('Down'), _('Left'), _('Right'), _('Front'), _('Back')]
-        for i in range(6):
-            liststore_faces.appendRow((QStandardItem(i), QStandardItem(face_names[i])))
-            filename = settings.theme.face[i].image
+    def fill_face_selector(self):
+        for i, (facekey, facename) in enumerate(self.facenames):
+            self.liststore_faces.appendRow((QStandardItem(i), QStandardItem(_(facename))))
+            filename = settings.theme.faces[facekey].image
             if filename.startswith('/'):
                 self.image_dirname = os.path.dirname(filename)
-        self.ui.listview_faces.setModel(liststore_faces)
         self.ui.listview_faces.setModelColumn(1)
         #XXX: workaround, listview_faces should automatically set to the correct width
         fm = QFontMetrics(self.ui.listview_faces.font())
-        width = max(fm.width(fn) for fn in face_names) + 8
+        width = max(fm.width(_(fn)) for fk, fn in self.facenames) + 8
         self.ui.listview_faces.setMaximumWidth(width)
-        self.ui.listview_faces.selectionModel().currentChanged.connect(self.on_listview_faces_currentChanged)
-        return liststore_faces
+        
+        active_face = 0
+        self.current_facekey = self.facenames[active_face][0]
+        index = self.liststore_faces.index(active_face, 1)
+        self.ui.listview_faces.setCurrentIndex(index)
         
     def fill_liststore_movekeys(self):
         for move, key in settings.draw.accels:
@@ -333,7 +342,7 @@ class PreferencesDialog (Dialog):
             
     def bind_reset_item(self, settings_key, button):
         def on_clicked():
-            delattr(settings, settings_key.format(self.active_face))
+            delattr(settings, settings_key.format(self.current_facekey))
         button.clicked.connect(on_clicked)
         
     @ staticmethod
@@ -348,16 +357,13 @@ class PreferencesDialog (Dialog):
         return accel_str
         
     def set_imagefile(self, imagefile):
-        index_icon = len(textures.stock_files) + 1
-        if not imagefile:
-            index = 0
-            icon = QIcon()
-        elif imagefile.startswith('/'):
+        index_icon = len(self.stockfiles)
+        if imagefile.startswith('/'):
             index = index_icon
             icon = QIcon(imagefile)
         else:
             try:
-                index = textures.stock_files.index(imagefile) + 1
+                index = self.stockfiles.index(imagefile)
             except ValueError:
                 index = 0
             icon = QIcon()
@@ -375,16 +381,12 @@ class PreferencesDialog (Dialog):
         elif key == 'draw.accels':
             self.liststore_movekeys.clear()
             self.fill_liststore_movekeys()
-        elif key == 'theme.face.{}.color'.format(self.active_face):
-            self.button_color.set_color(settings.theme.face[self.active_face].color)
-        elif key == 'theme.face.{}.image'.format(self.active_face):
-            self.set_imagefile(settings.theme.face[self.active_face].image)
-        elif key == 'theme.face.{}.mode':
-            imagemode = settings.theme.face[self.active_face].mode_nick
-            if imagemode == 'tiled':
-                self.ui.radiobutton_tiled.setChecked(True)
-            elif imagemode == 'mosaic':
-                self.ui.radiobutton_mosaic.setChecked(True)
+        elif key == 'theme.faces.{}.color'.format(self.current_facekey):
+            self.button_color.set_color(settings.theme.faces[self.current_facekey].color)
+        elif key == 'theme.faces.{}.image'.format(self.current_facekey):
+            self.set_imagefile(settings.theme.faces[self.current_facekey].image)
+        # Not needed:
+        #elif key == 'theme.faces.{}.mode'.format(self.current_facekey):
         self._ignore_changed = False
         
     ###
@@ -450,11 +452,12 @@ class PreferencesDialog (Dialog):
     ### appearance handlers ###
     
     def on_listview_faces_currentChanged(self, current):
-        self.active_face = current.row()
+        active_face = current.row()
+        self.current_facekey = self.facenames[active_face][0]
         self._ignore_changed = True
-        self.button_color.set_color(settings.theme.face[self.active_face].color)
-        self.set_imagefile(settings.theme.face[self.active_face].image)
-        imagemode = settings.theme.face[self.active_face].mode_nick
+        self.button_color.set_color(settings.theme.faces[self.current_facekey].color)
+        self.set_imagefile(settings.theme.faces[self.current_facekey].image)
+        imagemode = settings.theme.faces[self.current_facekey].mode_nick
         if imagemode == 'tiled':
             self.ui.radiobutton_tiled.setChecked(True)
         elif imagemode == 'mosaic':
@@ -465,30 +468,27 @@ class PreferencesDialog (Dialog):
         if self._ignore_changed:
             return
         self._ignore_changed = True
-        settings.theme.face[self.active_face].color = color
+        settings.theme.faces[self.current_facekey].color = color
         self._ignore_changed = False
         
     @Slot(int)
     def on_combobox_image_activated(self, index):
         if self._ignore_changed:
             return
-        if index == 0:
-            filename = ''
-        else:
-            try:
-                filename = textures.stock_files[index-1]
-            except IndexError:
-                filename = QFileDialog.getOpenFileName(self, _("Open Image"), self.image_dirname)
-                if isinstance(filename, (tuple, list)):
-                    filename = filename[0]
-                if filename == '':
-                    # canceled, set the old image
-                    filename = settings.theme.face[self.active_face].image
-                    self.set_imagefile(filename)
-                else:
-                    self.image_dirname = os.path.dirname(filename)
+        try:
+            filename = self.stockfiles[index]
+        except IndexError:
+            filename = QFileDialog.getOpenFileName(self, _("Open Image"), self.image_dirname)
+            if isinstance(filename, (tuple, list)):
+                filename = filename[0]
+            if filename == '':
+                # canceled, set the old image
+                filename = settings.theme.faces[self.current_facekey].image
+                self.set_imagefile(filename)
+            else:
+                self.image_dirname = os.path.dirname(filename)
         self._ignore_changed = True
-        settings.theme.face[self.active_face].image = filename
+        settings.theme.faces[self.current_facekey].image = filename
         self._ignore_changed = False
         
     @Slot(bool)
@@ -503,51 +503,69 @@ class PreferencesDialog (Dialog):
             return
         if checked:
             self._ignore_changed = True
-            settings.theme.face[self.active_face].mode_nick = mode
+            settings.theme.faces[self.current_facekey].mode_nick = mode
             self._ignore_changed = False
             
         
 class SelectModelDialog (Dialog):
-    response_ok = Signal(int, tuple, bool)
+    response_ok = Signal(str, tuple)
     
-    def __init__(self, parent):
-        Dialog.__init__(self, parent)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        
         self.ui = UI(self, 'model.ui')
         from .ui.model import retranslate
         retranslate(self)
         
-        for text in settings.game.type_range:
-            self.ui.combobox_model.addItem(_(text), text)
-        mtype = settings.game.type
-        self.ui.combobox_model.setCurrentIndex(mtype)
-        self.on_combobox_model_activated(mtype)
-        size = settings.game.size
-        width, height, depth = size
-        self.ui.spin_size1.setValue(width)
-        self.ui.spin_size2.setValue(height)
-        self.ui.spin_size3.setValue(depth)
+        self.mtypes = Model.cache_index['types']
+        self.modeldefs = [Model.cache_index['type'][mtype] for mtype in self.mtypes]
+        mtype_ranges = lambda mtype: tuple((min(vals), max(vals)) for vals in zip(*Model.cache_index['normsize'][mtype].keys()))
+        self.sranges = [mtype_ranges(mtype) for mtype in self.mtypes]
+        for i, mtype in enumerate(self.mtypes):
+            name = self.modeldefs[i]['name']
+            self.ui.combobox_model.addItem(_(name), mtype)
+        mtypeidx = self.mtypes.index(settings.game.type)
+        self.ui.combobox_model.setCurrentIndex(mtypeidx)
+        # Hide items after the dialog size is calculated to avoid later resizing
+        QTimer.singleShot(0, lambda: self.on_combobox_model_activated(mtypeidx))
         
     @Slot(int)
     def on_combobox_model_activated(self, index):
-        mtype = models[index].type
-        self.ui.label_width.setText({'Cube':_('Size:'), 'Tower':_('Basis:')}.get(mtype, _('Width:')))
-        self.ui.label_width.setEnabled(mtype in ['Cube', 'Tower', 'Brick'])
-        self.ui.label_heigth.setEnabled(mtype in ['Tower', 'Brick'])
-        self.ui.label_depth.setEnabled(mtype == 'Brick')
-        self.ui.spin_size1.setEnabled(mtype in ['Cube', 'Tower', 'Brick'])
-        self.ui.spin_size2.setEnabled(mtype in ['Tower', 'Brick'])
-        self.ui.spin_size3.setEnabled(mtype == 'Brick')
-        
+        mtype = self.mtypes[index]
+        size = settings.games[mtype].size
+        try:
+            size = Model.cache_index['normsize'][mtype][size]
+        except KeyError:
+            size = self.modeldefs[index]['defaultsize']
+        for label, spin, sizename, srange, svalue in zip_longest(
+                        (self.ui.label_width, self.ui.label_heigth, self.ui.label_depth),
+                        (self.ui.spin_size1, self.ui.spin_size2, self.ui.spin_size3),
+                        self.modeldefs[index]['sizenames'],
+                        self.sranges[index],
+                        size,
+                    ):
+            if label is None:
+                continue
+            label.setText(_(sizename) if sizename is not None else '')
+            spin.setVisible(sizename is not None)
+            if sizename is not None:
+                spin.setRange(*srange)
+                spin.setValue(svalue)
+                
     def on_response(self):
+        index = self.ui.combobox_model.currentIndex()
+        size_len = len(self.modeldefs[index]['defaultsize'])
+        spins = (self.ui.spin_size1, self.ui.spin_size2, self.ui.spin_size3)[:size_len]
         self.response_ok.emit(
-                self.ui.combobox_model.currentIndex(),
-                (self.ui.spin_size1.value(), self.ui.spin_size2.value(), self.ui.spin_size3.value()),
-                self.ui.checkbox_solved.isChecked())
+                self.mtypes[index],
+                tuple(s.value() for s in spins),
+                )
         
         
 class ProgressDialog (QProgressDialog):
-    def __init__(self, parent):
-        QProgressDialog.__init__(self, parent)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        
         self.canceled_ = False
         self.value_max = 10
         self.value = 0
@@ -582,6 +600,100 @@ class ProgressDialog (QProgressDialog):
         self.canceled_ = False
         self.reset()
         
+        
+class HelpDialog (Dialog):
+    
+    helpstrings = [
+            # The next strings form the text in the help dialog
+            _('Using the mouse to rotate the cube'),
+            _('Position the mouse cursor over the puzzle and you will see an arrow '
+              'that gives you a hint in which direction the slice under the mouse cursor will be rotated.'),
+            _('The left mouse button rotates a single slice of the cube in the direction of the arrow.'),
+            _('The right mouse button rotates a single slice of the cube against the direction of the arrow.'),
+            _('To rotate the whole cube instead of a single slice press the Ctrl key together with the mouse button.'),
+            
+            _('Using the keyboard to rotate the cube'),
+            _('Make sure the keyboard focus is on the cube area (e.g. click on the background of the cube). '
+              'The keys can be configured in the preferences dialog, the default is:'),
+            _('Moves the left, right, upper, down, front or back slice clockwise.'),
+            _('Moves a slice couterclockwise.'),
+            _('Moves the whole cube.'),
+            
+            _('Other keys and buttons'),
+            _('Mouse wheel – Zoom in/out'),
+            _('Arrow keys, Left mouse button on the background – Changes the direction of looking at the cube.'),
+            _('Moves keyboard focus to the sequence editor above the cube area '
+              'where you can edit the move sequence in the notation described below. '
+              'Hit enter when done.'),
+            
+            _('Notation for moves'),
+            _('All moves, however they are made, are displayed progressively above the cube area:'),
+            _('Moves the left, right, upper, down, front or back slice clockwise.'),
+            _('Moves a slice couterclockwise.'),
+            _('Moves the whole cube.'),
+            _('Moves the first, second or third slice from left clockwise. '
+              'The allowed numbers are in the range from 1 to the count of parallel slices. '
+              '"l1" is always the same as "l" '
+              'and for the classic 3×3×3-Cube "l2" is the same as "r2-" and "l3" is the same as "r-".'),
+            _('You can use a space to separate groups of moves.'),
+        ]
+    helpformat = '''
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
+<html>
+<head>
+<meta name="qrichtext" content="1" />
+<style type="text/css">p, li {{ white-space: pre-wrap; }}</style>
+</head>
+
+<body>
+
+<h3>{}</h3>
+<p>{}</p>
+<ul>
+<li>{}</li>
+<li>{}</li>
+<li>{}</li>
+</ul>
+
+<h3>{}</h3>
+<p>{}</p>
+<ul>
+<li>4,6,8,2,5,0 – {}</li>
+<li>Shift+4, … – {}</li>
+<li>Ctrl+4, … – {}</li>
+</ul>
+
+<h3>{}</h3>
+<ul>
+<li>{}</li>
+<li>{}</li>
+<li>Ctrl+L – {}</li>
+</ul>
+
+<h3>{}</h3>
+<p>{}</p>
+<ul>
+<li>l, r, u, d, f, b – {}</li>
+<li>l-, r-, u-, d-, f-, b- – {}</li>
+<li>L, R, U, D, F, B – {}</li>
+<li>l1, l2, l3 – {}</li>
+<li>{}</li>
+</ul>
+
+</body></html>
+    '''
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        
+        self.ui = UI(self, 'help.ui')
+        from .ui.help import retranslate
+        self.ui = retranslate(self)
+        
+        helpstrings = [html.escape(s) for s in self.helpstrings]
+        helptext = self.helpformat.format(*helpstrings)
+        self.ui.text_help.setHtml(helptext)
+        
+        
 def linkedtext_to_html(text):
     html = QTextDocumentFragment.fromPlainText(text).toHtml()
     html = re.sub(r'&lt;((?:http:|https:|text:).*?)\|&gt;', r'<a href="\1">', html)
@@ -591,18 +703,19 @@ def linkedtext_to_html(text):
 class UI:
     def __init__(self, toplevel, filename):
         self._toplevel = toplevel
-        from PyQt4 import uic
-        uic.loadUi(os.path.join(config.UI_DIR, filename), toplevel)
+        from PyQt5 import uic
+        uic.loadUi(os.path.join(config.UI_DIR, 'qt', filename), toplevel)
     def __getattr__(self, attrname):
         if attrname[0] == '_':
-            return object.__getattr__(self, attrname)
+            raise AttributeError('{!r} object has no attribute {!r}'.format('UI', attrname))
         else:
             return self._toplevel.findChild(QObject, attrname)
             
         
 class AboutDialog (QDialog):
-    def __init__(self, parent):
-        QDialog.__init__(self, parent)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        
         self.about = UI(self, 'about.ui')
         from .ui.about import retranslate
         retranslate(self)
@@ -622,16 +735,17 @@ class AboutDialog (QDialog):
         self.animation.setLoopCount(-1)
         
     def fill_header(self):
-        self.about.label_icon.setPixmap(QPixmap(os.path.join(config.PIXMAP_DIR, 'pybik-64')))
+        pixmap = QPixmap(config.APPICON_FILE)
+        self.about.label_icon.setPixmap(pixmap)
         self.about.label_appname.setText(_(config.APPNAME))
         self.about.label_version.setText(config.VERSION)
         self.about.label_description.setText(_(config.SHORT_DESCRIPTION))
         
     def fill_about_tab(self):
         self.about.label_copyright.setText(config.COPYRIGHT)
-        self.about.label_website.setText(
-                '<html><head/><body><p><a href="{}">{}</a></p></body></html>'
-                .format(config.WEBSITE, _('Pybik project website')))
+        html_text = '<html><head/><body><p><a href="{}">{}</a></p></body></html>'.format(
+                        config.WEBSITE, _('Pybik project website'))
+        self.about.label_website.setText(html_text)
         html_template = '''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
 <html><head><meta name="qrichtext" content="1" /><style type="text/css">
 p, li {{ white-space: pre-wrap; }}
@@ -661,15 +775,16 @@ p, li {{ white-space: pre-wrap; }}
                 language = langname_from_code(langcode, language)
                 yield language, persons
         for language, persons in sorted(gentranslators(), key=sortfunc):
-            language = Qt.escape(language)
+            language = html.escape(language)
             html_items = [html_language_template.format(language)]
             for name, link in sorted(persons, key=sortfunc):
-                name = Qt.escape(name)
+                name = html.escape(name)
                 html_items.append(html_person_template.format(link, name))
             html_p_items.append(html_p_template.format('<br/>'.join(html_items)))
-        html = html_template.format(''.join(html_p_items))
-        self.about.text_translators.setHtml(html)
-        self.about.text_translators.anchorClicked.connect(QDesktopServices.openUrl)
+        html_text = html_template.format(''.join(html_p_items))
+        self.about.text_translators.setHtml(html_text)
+        qdesktopservices_openurl = QDesktopServices.openUrl
+        self.about.text_translators.anchorClicked.connect(qdesktopservices_openurl)
         self.about.text_translators.viewport().installEventFilter(self)
         
     def fill_feedback_tab(self):
@@ -709,16 +824,16 @@ p, li {{ white-space: pre-wrap; }}
         self.animation.setKeyValueAt(0.54, smax)
         self.animation.setKeyValueAt(1., smin)
         
-    def showEvent(self, event):
+    def showEvent(self, unused_event):
         if self.animation.state() == QAbstractAnimation.Stopped:
             self.update_animation()
             self.scrollbar.hide()
             self.animation.start()
             
-    def resizeEvent(self, event):
+    def resizeEvent(self, unused_event):
         self.update_animation()
         
-    def eventFilter(self, obj, event):
+    def eventFilter(self, unused_obj, event):
         #assert obj == self.about.text_translators.viewport()
         if event.type() in [QEvent.MouseButtonPress, QEvent.Wheel]:
             self.animation.pause()
@@ -735,7 +850,6 @@ p, li {{ white-space: pre-wrap; }}
         if index == self.index_tab_license:
             self.about.text_license_short.setVisible(True)
             self.about.text_license_full.setVisible(False)
-        QTimer.singleShot(0, self.adjustSize)
         
     def _on_text_license_anchorClicked(self, url):
         if url.toString() == 'text:FULL_LICENSE_TEXT':
