@@ -1,9 +1,7 @@
 #-*- coding:utf-8 -*-
 # cython: profile=False
-# distutils: language = c++
 
-#  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009-2013  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2015  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -18,9 +16,6 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Ported from GNUbik
-# Original filename: glarea-common.c
-# Original copyright and license: 2003  John Darrington, GPL3+
 
 # although this file is compiled with Python3 syntax, Cython needs at least division from __future__
 from __future__ import print_function, division
@@ -38,22 +33,21 @@ from math import cos, sin, tan, atan2, pi as M_PI
 from OpenGL.GL import *
 #px-
 from OpenGL.GL.ARB.vertex_shader import glGetActiveAttribARB as glGetActiveAttrib
-#px/from glm cimport *
-from .glm import *
 
 #px/from _gldraw cimport *
 from .gldraw import *
 # pylint: enable=W0614,W0401
 
-from .debug import debug, DEBUG_DRAW, DEBUG_PICK, DEBUG_MSG, DEBUG_NOVSHADER, DEBUG_NOFSHADER, DEBUG_NOSHADER
+from .debug import debug, DEBUG, DEBUG_DRAW, DEBUG_PICK, DEBUG_MSG, DEBUG_NOVSHADER, DEBUG_NOFSHADER, DEBUG_NOSHADER
 
 #px/DEF SOURCEGLVERSION = 'GL'
 SOURCEGLVERSION = 'GL'
 
-debug('Importing module:', __name__)
-debug('  from package:', __package__)
-debug('  compiled:', __compiled)
-debug('  GL/GLES:', SOURCEGLVERSION)
+if DEBUG:
+    debug('Importing module:', __name__)
+    debug('  from package:', __package__)
+    debug('  compiled:', __compiled)
+    debug('  GL/GLES:', SOURCEGLVERSION)
 
 
 #px/cdef struct Terrain:
@@ -65,17 +59,20 @@ class terrain: pass     # pylint: disable=W0232, C0321, R0903
     #px+int height
 #px+cdef Terrain terrain
 
-#px/cdef struct Frustrum:
-class frustrum: pass    # pylint: disable=W0232, R0903
+#px/cdef struct Frustum:
+class frustum:
     #px+float fovy_angle  # field of view angle
     #px+float fovy_radius
     #px+float fovy_radius_zoom
     #px+double bounding_sphere_radius
     #px+bint multisample
-    #px+mat4 modelview_matrix
-    #px+mat4 projection_matrix
-    #px+mat4 picking_matrix
-#px+cdef Frustrum frustrum
+    #px/mat4 modelview_matrix
+    modelview_matrix = mat4()
+    #px/mat4 projection_matrix
+    projection_matrix = mat4()
+    #px/mat4 picking_matrix
+    picking_matrix = mat4()
+#px+cdef Frustum frustum
 
 #px/cdef struct Program:
 class program: pass
@@ -90,9 +87,15 @@ class program: pass
 #px+cdef Program program
 
 #px/cdef struct SelectionDebugPoints:
-class selection_debug_points: pass
-    #px+float modelview1[3], modelview2[3]
-    #px+int viewport1[2], viewport2[2]
+class selection_debug_points:
+    #px/float modelview1[3]
+    modelview1 = [0.] * 3
+    #px/float modelview2[3]
+    modelview2 = [0.] * 3
+    #px/int viewport1[2]
+    viewport1 = [0] * 2
+    #px/int viewport2[2]
+    viewport2 = [0] * 2
 #px+cdef SelectionDebugPoints selection_debug_points
 
 ### module state
@@ -104,21 +107,21 @@ def init_module():
     terrain.width = 1
     terrain.height = 1
     
-    frustrum.fovy_angle = 33.0  # field of view angle
-    frustrum.fovy_radius = 1 / tan(frustrum.fovy_angle * M_PI / 360.0)
-    frustrum.fovy_radius_zoom = frustrum.fovy_radius  # zoom == 1.
-    frustrum.bounding_sphere_radius = 1.
-    frustrum.multisample = 0
+    frustum.fovy_angle = 33.0  # field of view angle
+    frustum.fovy_radius = 1 / tan(frustum.fovy_angle * M_PI / 360.0)
+    frustum.fovy_radius_zoom = frustum.fovy_radius  # zoom == 1.
+    frustum.bounding_sphere_radius = 1.
+    frustum.multisample = 0
     # fill modelview_matrix
-    frustrum.modelview_matrix = mat4(1.)
+    matrix_set_identity(frustum.modelview_matrix)
     _update_modelview_matrix_translation()
-    # fill projection_matrix, see doc of glFrustum
-    frustrum.projection_matrix = mat4(1.)
-    frustrum.projection_matrix[2][3] = -1.
-    frustrum.projection_matrix[3][3] = 0.
+    # fill projection_matrix, see doc of _glFrustum
+    matrix_set_identity(frustum.projection_matrix)
+    frustum.projection_matrix[2][3] = -1.
+    frustum.projection_matrix[3][3] = 0.
     _update_projection_matrix()
     # fill picking_matrix
-    frustrum.picking_matrix = mat4(1.)
+    matrix_set_identity(frustum.picking_matrix)
     
     program.prog_render = 0
     program.prog_hud = 0
@@ -126,7 +129,7 @@ def init_module():
 
 #px/cdef void _update_modelview_matrix_translation():
 def _update_modelview_matrix_translation():
-    frustrum.modelview_matrix[3][2] = -frustrum.bounding_sphere_radius * (frustrum.fovy_radius_zoom + 1.)
+    frustum.modelview_matrix[3][2] = -frustum.bounding_sphere_radius * (frustum.fovy_radius_zoom + 1.)
     
 #px/cdef void _set_modelview_matrix_rotation(float radiansx, float radiansy):
 def _set_modelview_matrix_rotation(radiansx, radiansy):
@@ -134,8 +137,8 @@ def _set_modelview_matrix_rotation(radiansx, radiansy):
     #px+cdef float sx, sy, cx, cy
     #px+cdef float m00, m11, m12, m20
     
-    #px/M = &frustrum.modelview_matrix[0]
-    M = frustrum.modelview_matrix
+    #px/M = <vec4*>&frustum.modelview_matrix[0][0]
+    M = frustum.modelview_matrix
     
     sx = sin(radiansx/2)
     sy = sin(radiansy/2)
@@ -146,11 +149,10 @@ def _set_modelview_matrix_rotation(radiansx, radiansy):
     m11 = 2*cy*cy - 1.
     m12 = 2*sy*cy
     m20 = 2*sx*cx
-    # pylint: disable=C0321
+    # pylint: disable=C0321,C0326
     M[0][0] =  m00;         M[1][0] = 0.;   M[2][0] =  m20
     M[0][1] =  m12 * m20;   M[1][1] = m11;  M[2][1] = -m00 * m12
     M[0][2] = -m11 * m20;   M[1][2] = m12;  M[2][2] =  m00 * m11
-    # py lint: enable=C0321
     
 #px/cdef void _update_projection_matrix():
 def _update_projection_matrix():
@@ -160,36 +162,37 @@ def _update_projection_matrix():
     else:
         aspectx = terrain.width / terrain.height
         aspecty = 1.
-    # see doc of glFrustum
-    frustrum.projection_matrix[0][0] = frustrum.fovy_radius / aspectx
-    frustrum.projection_matrix[1][1] = frustrum.fovy_radius / aspecty
-    frustrum.projection_matrix[2][2] = -(frustrum.fovy_radius_zoom + 1.)
-    frustrum.projection_matrix[3][2] = -(frustrum.fovy_radius_zoom + 2.
-                    ) * frustrum.bounding_sphere_radius * frustrum.fovy_radius_zoom
+    # see doc of _glFrustum
+    frustum.projection_matrix[0][0] = frustum.fovy_radius / aspectx
+    frustum.projection_matrix[1][1] = frustum.fovy_radius / aspecty
+    frustum.projection_matrix[2][2] = -(frustum.fovy_radius_zoom + 1.)
+    frustum.projection_matrix[3][2] = -(frustum.fovy_radius_zoom + 2.
+                    ) * frustum.bounding_sphere_radius * frustum.fovy_radius_zoom
         
 #px/cdef void _set_picking_matrix(int x, int y):
 def _set_picking_matrix(x, y):
     # Set picking matrix, restrict drawing to one pixel of the viewport
-    # same as:  glLoadIdentity()
-    #           gluPickMatrix(x, y, 1, 1, viewport)
-    # same as:  glLoadIdentity()
-    #           glTranslatef(terrain.width - 2*x, terrain.height - 2*y, 0.)
-    #           glScalef(terrain.width, terrain.height, 1.0)
-    frustrum.picking_matrix[3][0] = terrain.width - 2*x
-    frustrum.picking_matrix[3][1] = terrain.height - 2*y
-    frustrum.picking_matrix[0][0] = terrain.width
-    frustrum.picking_matrix[1][1] = terrain.height
+    # same as:  _glLoadIdentity()
+    #           _gluPickMatrix(x, y, 1, 1, viewport)
+    # same as:  _glLoadIdentity()
+    #           _glTranslatef(terrain.width - 2*x, terrain.height - 2*y, 0.)
+    #           _glScalef(terrain.width, terrain.height, 1.0)
+    frustum.picking_matrix[3][0] = terrain.width - 2*x
+    frustum.picking_matrix[3][1] = terrain.height - 2*y
+    frustum.picking_matrix[0][0] = terrain.width
+    frustum.picking_matrix[1][1] = terrain.height
     
 #px/cdef void _set_picking_matrix_identity():
 def _set_picking_matrix_identity():
-    frustrum.picking_matrix[3][0] = 0.
-    frustrum.picking_matrix[3][1] = 0.
-    frustrum.picking_matrix[0][0] = 1.
-    frustrum.picking_matrix[1][1] = 1.
+    frustum.picking_matrix[3][0] = 0.
+    frustum.picking_matrix[3][1] = 0.
+    frustum.picking_matrix[0][0] = 1.
+    frustum.picking_matrix[1][1] = 1.
     
-def set_frustrum(bounding_sphere_radius, zoom):
-    frustrum.bounding_sphere_radius = bounding_sphere_radius
-    frustrum.fovy_radius_zoom = frustrum.fovy_radius / zoom
+def set_frustum(bounding_sphere_radius, zoom):
+    if bounding_sphere_radius > 0:
+        frustum.bounding_sphere_radius = bounding_sphere_radius
+    frustum.fovy_radius_zoom = frustum.fovy_radius / zoom
     _update_modelview_matrix_translation()
     _update_projection_matrix()
     
@@ -199,7 +202,7 @@ def set_background_color(red, green, blue):
     terrain.blue = blue
     
 def set_antialiasing(multisample):
-    frustrum.multisample = multisample
+    frustum.multisample = multisample
     
 def set_rotation_xy(x, y):
     x %= 360
@@ -255,9 +258,10 @@ def gl_init():
         print('  GL_SAMPLE_ALPHA_TO_COVERAGE:', glIsEnabled(GL_SAMPLE_ALPHA_TO_COVERAGE))
         #print('  GL_SAMPLE_ALPHA_TO_ONE:', glIsEnabled(GL_SAMPLE_ALPHA_TO_ONE))
         print('  GL_SAMPLE_COVERAGE:', glIsEnabled(GL_SAMPLE_COVERAGE))
+        _gl_print_integer('  GL_MAX_VERTEX_ATTRIBS:', GL_MAX_VERTEX_ATTRIBS)
     glClearColor(0., 0., 0., 1.)
     glEnable(GL_DEPTH_TEST)
-    glEnable(GL_CULL_FACE) # Rasterise only the exterior faces,  to speed things up
+    glEnable(GL_CULL_FACE)
     glCullFace(GL_BACK)
     glFrontFace(GL_CCW)
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
@@ -294,20 +298,21 @@ def gl_render():
         glUseProgram(program.prog_render)
         #px/IF SOURCEGLVERSION == 'GL':
         if True:
-            if frustrum.multisample:
+            if frustum.multisample:
                 glEnable(GL_MULTISAMPLE)
             else:
                 glDisable(GL_MULTISAMPLE)
         glClearColor(terrain.red, terrain.green, terrain.blue, 1.)
-        _gl_set_matrix(program.projection_location, frustrum.projection_matrix)
-        _gl_set_matrix(program.modelview_location, frustrum.modelview_matrix)
+        _gl_set_matrix(program.projection_location, frustum.projection_matrix)
+        _gl_set_matrix(program.modelview_location, frustum.modelview_matrix)
         glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
         gl_draw_cube()
     if DEBUG_DRAW:
         gl_draw_cube_debug()
         
 def gl_render_select_debug():
-    #px+cdef GLfloat selectdata[12]
+    #px/cdef GLfloat selectdata[12]
+    selectdata = [0.] * 12
     selectdata[0] = selection_debug_points.modelview1[0]
     selectdata[1] = selection_debug_points.modelview1[1]
     selectdata[2] = selection_debug_points.modelview1[2]
@@ -321,7 +326,7 @@ def gl_render_select_debug():
     selectdata[10] = selection_debug_points.viewport2[1] / terrain.height * 2 - 1
     selectdata[11] = 0
     #px/gl_draw_select_debug(&selectdata[0], sizeof(selectdata), program.prog_hud)
-    gl_draw_select_debug(selectdata, sizeof(selectdata), program.prog_hud)
+    gl_draw_select_debug(selectdata, 0, program.prog_hud)
     
 
 ### picking functions
@@ -333,9 +338,9 @@ def _gl_render_pick():
     if True:
         glDisable(GL_MULTISAMPLE)
     glClearColor(0., 0., 0., 1.)
-    _gl_set_matrix(program.picking_location, frustrum.picking_matrix)
-    _gl_set_matrix(program.projection_pick_location, frustrum.projection_matrix)
-    _gl_set_matrix(program.modelview_pick_location, frustrum.modelview_matrix)
+    _gl_set_matrix(program.picking_location, frustum.picking_matrix)
+    _gl_set_matrix(program.projection_pick_location, frustum.projection_matrix)
+    _gl_set_matrix(program.modelview_pick_location, frustum.modelview_matrix)
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
     gl_pick_cube()
     
@@ -352,18 +357,19 @@ def gl_pick_polygons(x, y):
     index = pixel[0]<<4 | pixel[1] | pixel[2]>>4
     return index
     
-#px/cdef void _modelview_to_viewport(vvect, int *mvect):
+#px/cdef void _modelview_to_viewport(float *vvect, int *mvect):
 def _modelview_to_viewport(vvect, mvect):
     #px+cdef vec4 *M
     #px+cdef vec4 *P
     #px+cdef float u0, u1, u2, v0, v1, v3
     
-    #px/M = &frustrum.modelview_matrix[0]
-    M = frustrum.modelview_matrix
-    #px/P = &frustrum.projection_matrix[0]
-    P = frustrum.projection_matrix
+    #px/M = <vec4*>&frustum.modelview_matrix[0][0]
+    M = frustum.modelview_matrix
+    #px/P = <vec4*>&frustum.projection_matrix[0][0]
+    P = frustum.projection_matrix
     
     # u = M^T * vvect
+    #assert M[1][0] == 0
     u0 = M[0][0]*vvect[0] + M[1][0]*vvect[1] + M[2][0]*vvect[2] + M[3][0]
     u1 = M[0][1]*vvect[0] + M[1][1]*vvect[1] + M[2][1]*vvect[2] + M[3][1]
     u2 = M[0][2]*vvect[0] + M[1][2]*vvect[1] + M[2][2]*vvect[2] + M[3][2]
@@ -378,16 +384,58 @@ def _modelview_to_viewport(vvect, mvect):
     mvect[0] = int((v0 / v3 + 1) / 2 * terrain.width)
     mvect[1] = int((v1 / v3 + 1) / 2 * terrain.height)
     
-def get_cursor_angle(point1, point2):
-    '''The result is the angle (on the screen)
-    at which the mouse cursor needs to be drawn.'''
+#px/cdef void _viewport_to_modelview(int *vvect, float *mvect):
+def _viewport_to_modelview(vvect, mvect):
+    #px+cdef vec4 *MT
+    #px+cdef vec4 *P
+    #px+cdef float u0, u1, u2, u3, v0, v1, v2
+    
+    v0 = vvect[0] / terrain.width * 2 - 1
+    v1 = vvect[1] / terrain.height * 2 - 1
+    v2 = 0
+    #v3 = 1
+    
+    # P * P^-1 = I
+    # a 0 0 0     A 0 0 0     aA 0  0  0      E = -eD/c
+    # 0 b 0 0  *  0 B 0 0  =  0  bB 0  0 ,    
+    # 0 0 e c     0 0 0 D     0  0  cC 0      
+    # 0 0 d 0     0 0 C E     0  0  0  dD     
+    #px/P = <vec4*>&frustum.projection_matrix[0][0]
+    P = frustum.projection_matrix
+    # u = P^-1 * v
+    u0 = 1 / P[0][0] * v0
+    u1 = 1 / P[1][1] * v1
+    u2 = 1 / P[2][3]
+    #assert u2 == -1
+    u3 = - P[2][2] / P[3][2] / P[2][3]
+    
+    # MT * MT^-1 = I
+    # a b c 0     a 0 f 0     1 0 0 0    U = -cz
+    # 0 d e 0  *  b d g 0  =  0 1 0 0,   V = -ez
+    # f g h 0     c e h 0     0 0 1 0    W = -hz
+    # 0 0 z 1     U V W 1     0 0 0 1
+    #px/MT = <vec4*>&frustum.modelview_matrix[0][0]
+    MT = frustum.modelview_matrix
+    # v = M^-1 * u
+    v0 = MT[0][0]*u0 + MT[0][1]*u1 + MT[0][2]*u2 - MT[0][2] * MT[3][2]*u3
+    v1 = MT[1][0]*u0 + MT[1][1]*u1 + MT[1][2]*u2 - MT[1][2] * MT[3][2]*u3
+    v2 = MT[2][0]*u0 + MT[2][1]*u1 + MT[2][2]*u2 - MT[2][2] * MT[3][2]*u3
+    #v3 = u3
+    
+    mvect[0] = v0 / u3
+    mvect[1] = v1 / u3
+    mvect[2] = v2 / u3
+    
+def get_cursor_angle(xi, yi, d):
     #px+cdef float angle
-    #px+cdef int i, x, y
+    #px+cdef int i
+    #px+cdef float x, y
+    selection_debug_points.viewport1[0] = xi
+    selection_debug_points.viewport1[1] = yi
+    _viewport_to_modelview(selection_debug_points.viewport1, selection_debug_points.modelview1)
     for i in range(3):
-        selection_debug_points.modelview1[i] = point1[i]
-        selection_debug_points.modelview2[i] = point2[i]
-    _modelview_to_viewport(point1, selection_debug_points.viewport1)
-    _modelview_to_viewport(point2, selection_debug_points.viewport2)
+        selection_debug_points.modelview2[i] = d[i] + selection_debug_points.modelview1[i]
+    _modelview_to_viewport(selection_debug_points.modelview2, selection_debug_points.viewport2)
     x = selection_debug_points.viewport1[0] - selection_debug_points.viewport2[0]
     y = selection_debug_points.viewport1[1] - selection_debug_points.viewport2[1]
     angle = atan2(x, y) * 180.0 / M_PI
@@ -406,7 +454,7 @@ def _gl_print_shader_log(shader):
     if log_len > 0:
         print('==== Error compiling shader:')
         #px/glGetShaderInfoLog(shader, 1023, &length, log)
-        log = glGetShaderInfoLog(shader, 1023)
+        log = glGetShaderInfoLog(shader)
         print(log.decode('utf-8').rstrip())
         print('====')
     else:
@@ -422,7 +470,7 @@ def _gl_print_program_log(program):
     if log_len > 0:
         print('==== Error linking shader program:')
         #px/glGetProgramInfoLog(program, 1023, &length, log)
-        log = glGetProgramInfoLog(program, 1023)
+        log = glGetProgramInfoLog(program)
         print(log.decode('utf-8').rstrip())
         print('====')
     else:
@@ -440,7 +488,7 @@ def _gl_create_compiled_shader(shadertype, source):
         return 0
     #px+pchar = <const_GLchar_ptr><char*>source
     #px/glShaderSource(shader, 1, &pchar, NULL)
-    glShaderSource(shader, source)
+    glShaderSource(shader, source.decode('utf-8')) # PyOpenGL needs str currently
     glCompileShader(shader)
     #px/glGetShaderiv(shader, GL_COMPILE_STATUS, &compile_status)
     compile_status = glGetShaderiv(shader, GL_COMPILE_STATUS)
@@ -477,18 +525,18 @@ def _gl_print_program_info(program):
     print('active attributes:')
     for i in range(program_info[GL_ACTIVE_ATTRIBUTES]):
         #px/glGetActiveAttrib(program, i, 1023, &alength, &asize, &atype, aname)
-        aname, asize, atype = glGetActiveAttrib(program, i); alength = '?'
+        aname, asize, atype = glGetActiveAttrib(program, i); alength = len(aname)
         location = glGetAttribLocation(program, aname)
         print('  {}, {}: length={} size={} type={} location={}'.format(i, aname, alength, asize, atype, location))
     print('active uniforms:')
     for i in range(program_info[GL_ACTIVE_UNIFORMS]):
         #px/glGetActiveUniform(program, i, 1023, &alength, &asize, &atype, aname)
-        aname, asize, atype = glGetActiveUniform(program, i); alength = '?'
+        aname, asize, atype = glGetActiveUniform(program, i); alength = len(aname)
         location = glGetUniformLocation(program, aname)
         print('  {}, {}: length={} size={} type={} location={}'.format(i, aname, alength, asize, atype, location))
         
-#px/cdef GLuint _gl_create_program(bytes vertex_source, bytes fragment_source, list attributes):
-def _gl_create_program(vertex_source, fragment_source, attributes):
+#px/cdef GLuint _gl_create_program(bytes vertex_source, bytes fragment_source, int attrib_location, list attributes):
+def _gl_create_program(vertex_source, fragment_source, attrib_location, attributes):
     #px+cdef GLuint vertex_shader = 0, fragment_shader = 0
     #px+cdef GLuint program
     #px+cdef GLint link_status
@@ -500,13 +548,20 @@ def _gl_create_program(vertex_source, fragment_source, attributes):
     if not DEBUG_NOVSHADER:
         debug('  creating vertex shader')
         vertex_shader = _gl_create_compiled_shader(GL_VERTEX_SHADER, vertex_source)
-        glAttachShader(program, vertex_shader)
+        if vertex_shader:
+            glAttachShader(program, vertex_shader)
+        else:
+            print('error: no vertex shader')
     if not DEBUG_NOFSHADER:
         debug('  creating fragment shader')
         fragment_shader = _gl_create_compiled_shader(GL_FRAGMENT_SHADER, fragment_source)
-        glAttachShader(program, fragment_shader)
-    for index, name in attributes:
-        glBindAttribLocation(program, index, name)
+        if fragment_shader:
+            glAttachShader(program, fragment_shader)
+        else:
+            print('error: no fragment shader')
+    for index, name in enumerate(attributes):
+        if name is not None:
+            glBindAttribLocation(program, attrib_location+index, name)
     glLinkProgram(program)
     #px/glGetProgramiv(program, GL_LINK_STATUS, &link_status)
     link_status = glGetProgramiv(program, GL_LINK_STATUS)
@@ -527,12 +582,13 @@ def gl_create_render_program(vertex_source, fragment_source):
     if program.prog_render > 0:
         glDeleteProgram(program.prog_render)
     attributes = [
-                  (VERTEX_ATTRIB_LOCATION, b'vertex_attr'),
-                  (NORMAL_ATTRIB_LOCATION, b'normal_attr'),
-                  (COLOR_ATTRIB_LOCATION, b'color_attr'),
-                  (TEXCOORD_ATTRIB_LOCATION, b'texcoord_attr'),
+                  b'vertex_attr',
+                  b'normal_attr',
+                  b'color_attr',
+                  b'texcoord_attr',
+                  b'barycentric_attr',
                  ]
-    program.prog_render = _gl_create_program(vertex_source, fragment_source, attributes)
+    program.prog_render = _gl_create_program(vertex_source, fragment_source, ATTRIB_LOCATION, attributes)
     if program.prog_render > 0:
         glUseProgram(program.prog_render)
         location = glGetUniformLocation(program.prog_render, b'tex')
@@ -547,11 +603,8 @@ def gl_create_hud_program(vertex_source, fragment_source):
     #px+cdef GLint location
     if program.prog_hud > 0:
         glDeleteProgram(program.prog_hud)
-    attributes = [
-                  (VERTEX_ATTRIB_LOCATION, b'vertex_attr'),
-                  (COLOR_ATTRIB_LOCATION, b'color_attr'),
-                 ]
-    program.prog_hud = _gl_create_program(vertex_source, fragment_source, attributes)
+    attributes = [b'vertex_attr', None, b'color_attr']
+    program.prog_hud = _gl_create_program(vertex_source, fragment_source, ATTRIB_LOCATION, attributes)
     if program.prog_hud > 0:
         glUseProgram(program.prog_hud)
     
@@ -559,9 +612,8 @@ def gl_create_hud_program(vertex_source, fragment_source):
 def gl_create_pick_program(vertex_source, fragment_source):
     if program.prog_pick > 0:
         glDeleteProgram(program.prog_pick)
-    attributes = [(VERTEX_ATTRIB_LOCATION, b'vertex_attr'),
-                  (COLOR_ATTRIB_LOCATION, b'color_attr')]
-    program.prog_pick = _gl_create_program(vertex_source, fragment_source, attributes)
+    attributes = [b'vertex_attr', b'color_attr']
+    program.prog_pick = _gl_create_program(vertex_source, fragment_source, PICKATTRIB_LOCATION, attributes)
     if program.prog_pick > 0:
         glUseProgram(program.prog_pick)
         program.picking_location = glGetUniformLocation(program.prog_pick, b'picking')
