@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
 
-#  Copyright © 2012-2013  B. Clausius <barcc@gmx.de>
+#  Copyright © 2012-2013, 2015  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -22,22 +22,17 @@ import unicodedata
 
 
 symbols = [
-    #('ATOM SYMBOL',         'DejaVu-Sans-Oblique',  300, '+0+3'),
-    ('ATOM SYMBOL',         'FreeSerif-Mittel',     256, '+0-6'),
-    #('BEAMED EIGHTH NOTES', 'DejaVu-Sans-Oblique',  256, '+0-6'),
-    ('BEAMED EIGHTH NOTES', 'FreeSerif-Mittel',     256, '+0+16'),
+    # list fonts: convert -list font
+    ('ATOM SYMBOL',         'FreeSerif',            256, '+0-16'),
+    ('BEAMED EIGHTH NOTES', 'FreeSerif',            256, '+0+5'),
     ('BLACK SMILING FACE',  'DejaVu-Sans-Oblique',  256, '-1+5'),
     ('PEACE SYMBOL',        'DejaVu-Sans-Oblique',  300, '-2+5'),
     ('SHAMROCK',            'DejaVu-Sans-Oblique',  300, '-1+5'),
     ('SKULL AND CROSSBONES','DejaVu-Sans-Oblique',  300, '-1+5'),
-    #('SKULL AND CROSSBONES','FreeSerif-Mittel',     300, '+0+0'),
-    #('SNOWFLAKE',           'DejaVu-Sans-Oblique',  300, '-1+5'),
-    ('SNOWFLAKE',           'FreeSerif-Mittel',     320, '+0+14'),
+    ('SNOWFLAKE',           'FreeSerif',            320, '+0+14'),
     ('WHITE SMILING FACE',  'DejaVu-Sans-Oblique',  256, '-1+5'),
-    #('WHITE SUN WITH RAYS', 'DejaVu-Sans-Oblique',  300, '+0+4'),
-    ('WHITE SUN WITH RAYS', 'FreeMono-Standard',    390, '+0+0'),
+    ('WHITE SUN WITH RAYS', 'FreeMono',             390, '+0+0'),
     ('YIN YANG',            'DejaVu-Sans-Oblique',  300, '-2+5'),
-    #'FOUR LEAF CLOVER',
     #'ANTICLOCKWISE GAPPED CIRCLE ARROW',
     #'CLOCKWISE GAPPED CIRCLE ARROW',
 ]
